// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

#include "fragmentVersionCentroid.h"

#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
			_centroid in highp vec2 uv0;
			_centroid in highp vec2 uv1;
		#else
			_centroid in vec2 uv0;
			_centroid in vec2 uv1;
		#endif
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		varying vec2 uv0;
		varying vec2 uv1;
	#endif
#endif

varying vec4 color;
varying vec3 wpos;
varying vec3 position;

#ifdef FOG
varying vec4 fogColor;
#endif

#include "uniformShaderConstants.h"
#include "util.h"
uniform float TIME;
uniform float RENDER_DISTANCE;
uniform vec2 FOG_CONTROL;
uniform vec4 FOG_COLOR;

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;
LAYOUT_BINDING(2) uniform sampler2D TEXTURE_2;

vec3 acesFilm(vec3 x){
const float a = 2.65f;
const float b = 0.03f;
const float c = 2.53f;
const float d = 0.59f;
const float e = 0.14f;
return clamp((x*(a*x+b))/(x*(c*x+d)+e),0.0,1.0);}

vec3 saturate(vec3 rgb, float adjustment){
const vec3 W = vec3(0.2125, 0.7154, 0.0721);
vec3 intensity = vec3(dot(rgb, W));
return mix(intensity, rgb, adjustment);
}

float inrect(vec2 pos, float x1, float y1, float x2, float y2, float focus){
 return min(1.0, max(min(min(pos.x - x1, x2 - pos.x), min(pos.y - y1, y2 - pos.y)), 0.0) / focus);
}

float playershadow(){
vec3 lookvector=wpos;
lookvector.x*=2.0;
 vec3 pos = lookvector + vec3(0.4, 0.4, 0.4); 
 vec3 dir = vec3(-1.0, (1.25) * 0.31, 0.0);
 float factor = 1.0;
 if (pos.x < 0.2){
  factor = max(0.0, pos.x / 0.4 + 0.5);
 }
pos += dir * pos.x;
 float focus = .04;
 float footwalk = sin((position.x - lookvector.x) * 2.0 + (position.z - lookvector.z) * 2.0);
float handswalk = sin((position.x - lookvector.x) * 2.0 + (position.z - lookvector.z)) * .5;
pos.yz -= vec2(.2, .4);
 float body = max(inrect(pos.yz, -1.5 + footwalk*.4, -0.25, 0.75, .1, focus), inrect(pos.yz, -1.5 - footwalk*.4, -.1, 0.75, 0.25, focus));
 float hands = max(inrect(pos.yz, -0.5 + footwalk * .1, -0.5, 0.25, .1, focus), inrect(pos.yz, -0.5 - footwalk * 0.1, -.1, 0.25, 0.5, focus));
 return min(1.0, max(body, hands)) * factor;
}

highp float hash(
	highp float u){
return fract(sin(u)*43758.5453);
	}

highp float noise( highp vec2 x ){
	highp vec2 p = floor(x);
	highp vec2 f = fract(x);
    f = f*f*(3.0-2.0*f);

    highp float n = p.x + p.y*57.0;
return mix(mix( hash(n+  0.0), hash(n+  1.0),f.x),
	mix( hash(n+ 57.0), hash(n+ 58.0),f.x),f.y);
}

void main()
{
#ifdef BYPASS_PIXEL_SHADER
	gl_FragColor = vec4(0, 0, 0, 0);
	return;
#else 

#if USE_TEXEL_AA
	vec4 diffuse = texture2D_AA(TEXTURE_0, uv0); 
   vec4 limits = texture2D_AA(TEXTURE_1, vec2(0,1));
#else
	vec4 diffuse = texture2D(TEXTURE_0, uv0);
   vec4 limits = texture2D(TEXTURE_1, vec2(0,1));
#endif
	
#ifdef SEASONS_FAR
	diffuse.a = 1.0;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
	#define ALPHA_THRESHOLD 0.05
	#else
	#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif
	
vec4 inColor = color;

#if defined(BLEND)
	diffuse.a *= inColor.a;
#endif

#if !defined(ALWAYS_LIT)
	diffuse *= texture2D( TEXTURE_1, uv1 );
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = inColor.a;
	#endif
	
	diffuse.rgb *= inColor.rgb;
#else
	vec2 uv = inColor.xy;
	diffuse.rgb *= mix(vec3(1.0,1.0,1.0), texture2D( TEXTURE_2, uv).rgb*2.0, inColor.b);
	diffuse.rgb *= inColor.aaa;
	diffuse.a = 1.0;
#endif

bool unwater = FOG_CONTROL.x==0.0;
float rain = smoothstep(0.66, 0.3, FOG_CONTROL.x);
bool water = (color.b>color.r);
bool nether = (FOG_COLOR.r>FOG_COLOR.b && FOG_COLOR.r<0.5 && FOG_COLOR.b<0.2);

diffuse.rgb *= vec3(1.8,1.7,1.6);
if(uv1.y<0.875){diffuse.rgb *= vec3(0.98)+uv1.x*vec3(0.03);}
if(uv1.y<0.873){diffuse.rgb *= vec3(0.94)+uv1.x*vec3(0.02);}
if(uv1.y<0.871){diffuse.rgb *= vec3(0.88)+uv1.x*vec3(0.12);}
if(uv1.y<0.869){diffuse.rgb *= vec3(0.8)+uv1.x*vec3(0.2);}

if(color.a != 0.0){
if(color.g<0.64){diffuse.rgb *= vec3(0.98)+uv1.x*vec3(0.02);}
if(color.g<0.63){diffuse.rgb *= vec3(0.94)+uv1.x*vec3(0.06);}
if(color.g<0.62){diffuse.rgb *= vec3(0.88)+uv1.x*vec3(0.12);}
if(color.g<0.61){diffuse.rgb *= vec3(0.8)+uv1.x*vec3(0.2);}
}

//TimeCycle
vec4 detc = texture2D(TEXTURE_1, vec2(0.0,1.0));
float day = pow(min(max(1.1-pow(detc.r,10.0),0.0),1.0),3.0);
float night = min(pow(1.0-detc.r,2.0)*2.0,1.0);

diffuse.rgb *= vec3(1.7);
vec3 colorN = vec3(0.1,0.3,0.8);
colorN = mix(colorN, vec3(1.0), uv1.x);
diffuse.rgb *= mix(vec3(1.0),colorN,night);

float area = smoothstep( .885, .875, uv1.y);
vec3 kena = mix(vec3(0.58), vec3(1.2), uv1.x);
diffuse.rgb *= mix(vec3(1.0), kena, area);

if(unwater==false){
vec3 torch = vec3(2.7,0.6,0.7);
diffuse.rgb += diffuse.rgb* (torch* 1.3)* pow(uv1.x, 6.0)* (1.7-uv1.y);}

if(unwater== true){
vec3 sea = vec3(0.2,0.2,1.3);
vec3 source = vec3(0.0,0.0,3.0);
diffuse.rgb += sea*uv1.x;
diffuse.rgb += diffuse.rgb* (source)* pow(uv1.x, 60.0)* (1.0-uv1.y);}

#if defined(BLEND) || defined(FOG)
if(water==true){
diffuse.a = 0.786;//Opacitiy

diffuse.rgb *= vec3(0.0);//Cleared texture base

//Color in Day
vec3 clor = mix(vec3(0.1,0.3,0.8), vec3(1.8), clamp(length(wpos)/RENDER_DISTANCE*2.0,0.0,3.0));

//Color in Night
vec3 clorN = mix(vec3(0.19,0.0,0.68), vec3(0.5), clamp(length(wpos)/RENDER_DISTANCE*1.0,0.0,1.0));

//Color in rain
vec3 clorR = mix(vec3(0.1,0.1,0.1), vec3(4.0), clamp(length(wpos)/RENDER_DISTANCE*2.0,0.0,2.0));

//activators
vec3 lor = mix(clor,clorN, night);
diffuse.rgb += mix(lor,clorR, rain);

highp float time = TIME*1.33;
highp float wwave = noise(time+position.xz+position.zx+position.xy);
diffuse.rgb += wwave*0.098833;

if(nether==false){
if(FOG_COLOR.r*1.0>FOG_COLOR.b){

diffuse.rgb += clamp(1.0-length(-wpos.z*10.0)/RENDER_DISTANCE*1.0,0.0,1.0)*vec3(2.0,1.78,0.0)*length(-wpos.x)/RENDER_DISTANCE*2.0;}}

}
#endif

float radius = clamp(length(-wpos *0.5)/RENDER_DISTANCE *2.0,0.0,1.0);
vec3 fog = mix(diffuse.rgb, FOG_COLOR.rgb *vec3(0.96), radius);
diffuse.rgb = fog;

if(unwater==true){
diffuse.rgb *= vec3(0.4,0.5,0.63);

}

vec2 render = uv1;
render.y -= playershadow()*0.11;
diffuse *= texture2D(TEXTURE_1,render);

diffuse.rgb = acesFilm(diffuse.rgb);
diffuse.rgb = saturate(diffuse.rgb,1.5);

if(nether==false){
if(FOG_COLOR.r*1.0>FOG_COLOR.b){
	
vec3 beam = noise(vec2(wpos.z/wpos.y))*vec3(1.3,0.7,0.17);

vec3 sbeamc = mix(vec3(0.0),beam,clamp(length(wpos)/RENDER_DISTANCE*0.8,0.0,1.0));

diffuse.rgb += sbeamc;

}}

#ifdef FOG
if(unwater==true){
	diffuse.rgb = mix( diffuse.rgb, fogColor.rgb, fogColor.a );}
#endif

	gl_FragColor = diffuse;
	
#endif // BYPASS_PIXEL_SHADER
}